@@ -123,6 +123,7 @@ transfer.  For example:
 
 
 """
+
 import concurrent.futures
 import functools
 import logging
@@ -135,7 +136,7 @@ import string
 import threading
 
 from botocore.compat import six  # noqa: F401
-from botocore.exceptions import IncompleteReadError
+from botocore.exceptions import IncompleteReadError, ResponseStreamingError
 from botocore.vendored.requests.packages.urllib3.exceptions import (
     ReadTimeoutError,
 )
@@ -144,7 +145,7 @@ import s3transfer.compat
 from s3transfer.exceptions import RetriesExceededError, S3UploadFailedError
 
 __author__ = 'Amazon Web Services'
-__version__ = '0.8.0'
+__version__ = '0.11.3'
 
 
 class NullHandler(logging.Handler):
@@ -624,6 +625,7 @@ class MultipartDownloader:
                     OSError,
                     ReadTimeoutError,
                     IncompleteReadError,
+                    ResponseStreamingError,
                 ) as e:
                     logger.debug(
                         "Retrying exception caught (%s), "
@@ -681,7 +683,6 @@ class TransferConfig:
 
 
 class S3Transfer:
-
     ALLOWED_DOWNLOAD_ARGS = [
         'VersionId',
         'SSECustomerAlgorithm',
@@ -716,6 +717,9 @@ class S3Transfer:
 
     def __init__(self, client, config=None, osutil=None):
         self._client = client
+        self._client.meta.events.register(
+            'before-call.s3.*', self._update_checksum_context
+        )
         if config is None:
             config = TransferConfig()
         self._config = config
@@ -723,6 +727,13 @@ class S3Transfer:
             osutil = OSUtils()
         self._osutil = osutil
 
+    def _update_checksum_context(self, params, **kwargs):
+        request_context = params.get("context", {})
+        checksum_context = request_context.get("checksum", {})
+        if "request_algorithm" in checksum_context:
+            # Force request checksum algorithm in the header if specified.
+            checksum_context["request_algorithm"]["in"] = "header"
+
     def upload_file(
         self, filename, bucket, key, callback=None, extra_args=None
     ):
@@ -813,8 +824,8 @@ class S3Transfer:
         for kwarg in actual:
             if kwarg not in allowed:
                 raise ValueError(
-                    "Invalid extra_args key '%s', "
-                    "must be one of: %s" % (kwarg, ', '.join(allowed))
+                    f"Invalid extra_args key '{kwarg}', "
+                    f"must be one of: {', '.join(allowed)}"
                 )
 
     def _ranged_download(
@@ -841,6 +852,7 @@ class S3Transfer:
                 OSError,
                 ReadTimeoutError,
                 IncompleteReadError,
+                ResponseStreamingError,
             ) as e:
                 # TODO: we need a way to reset the callback if the
                 # download failed.
